<?php

defined('_VALID_CALL') or die('Direct Access is not allowed.');

require_once _SRV_WEBROOT . _SRV_WEB_PLUGINS. 'xt_klarna_kco/classes/constants.php';

//error_log('in '.__FILE__);

class klarna_kco
{
    // klarna backend urls
    const EU_TEST_URL_BACKEND   = "https://orders.playground.eu.portal.klarna.com";
    const EU_URL_BACKEND        = "https://orders.eu.portal.klarna.com";

    private $_xt_order_id = false;
    private static $_instance = false;

    private function __construct()
    {
        return $this;
    }

    public static function getInstance()
    {
        if(!is_object(self::$_instance))
        {
            self::$_instance = new klarna_kco();
        }
        return self::$_instance;
    }



    public function setXtOrderId($xt_order_id)
    {
        $this->_xt_order_id = $xt_order_id;
        return $this;
    }
    public function getXtOrderId()
    {
        return $this->_xt_order_id;
    }

    public static function getBackendBaseUrl()
    {
        if(!defined('_KLARNA_CONFIG_TESTMODE'))
            return self::EU_TEST_URL_BACKEND;
        if(constant('_KLARNA_CONFIG_TESTMODE')==1)
        {
            return self::EU_TEST_URL_BACKEND;
        }
        return self::EU_URL_BACKEND;
    }

    public static function getBaseUrl()
    {
        if(!defined('_KLARNA_CONFIG_TESTMODE'))
            return \Klarna\Rest\Transport\ConnectorInterface::EU_TEST_BASE_URL;
        if(constant('_KLARNA_CONFIG_TESTMODE')==1)
        {
            return \Klarna\Rest\Transport\ConnectorInterface::EU_TEST_BASE_URL;
        }
        return \Klarna\Rest\Transport\ConnectorInterface::EU_BASE_URL;
    }

    public function getConnector($store_id = null)
    {
        global $db;

        $mid = constant('_KLARNA_CONFIG_MID');
        $pwd = constant('_KLARNA_CONFIG_PWD');

        if(constant('_KLARNA_CONFIG_TESTMODE') == 1)
        {
            $mid = constant('_KLARNA_CONFIG_MID_TEST');
            $pwd = constant('_KLARNA_CONFIG_PWD_TEST');
        }

        if((is_object($this) && $this->_xt_order_id) || $store_id)
        {
            global $db;
            if(empty($store_id))
                $store_id = $db->GetOne("SELECT o.shop_id FROM ".TABLE_ORDERS." o LEFT JOIN ".TABLE_MANDANT_CONFIG." s ON s.shop_id = o.shop_id WHERE o.orders_id=?", array($this->_xt_order_id));
            if($store_id)
            {
                $key_mid = '_KLARNA_CONFIG_MID';
                $key_pwd = '_KLARNA_CONFIG_PWD';

                $test_mode = $db->GetOne("SELECT config_value FROM ".TABLE_CONFIGURATION_MULTI.$store_id. " WHERE config_key = '_KLARNA_CONFIG_TESTMODE' ");
                if($test_mode)
                {
                    $key_mid = '_KLARNA_CONFIG_MID_TEST';
                    $key_pwd = '_KLARNA_CONFIG_PWD_TEST';
                }

                $mid = $db->GetOne("SELECT config_value FROM ".TABLE_CONFIGURATION_MULTI.$store_id. " WHERE config_key = '".$key_mid."' ");
                $pwd = $db->GetOne("SELECT config_value FROM ".TABLE_CONFIGURATION_MULTI.$store_id. " WHERE config_key = '".$key_pwd."' ");
            }
            else {
                throw new Exception('No store found for xt-order ['.$this->_xt_order_id.']');
            }
        }

        $module_version = $db->GetOne("SELECT version FROM ".TABLE_PLUGIN_PRODUCTS." p WHERE p.code='xt_klarna_kco'");
        $userAgent = \Klarna\Rest\Transport\UserAgent::createDefault();
        $userAgent->setField('Shop','XT-Commerce',_SYSTEM_VERSION);
        $userAgent->setField('Module','xt_klarna_kco',$module_version);


        $connector = \Klarna\Rest\Transport\Connector::create(
            $mid,
            $pwd,
            self::getBaseUrl(),
            $userAgent
        );
        return $connector;
    }

    public static function setKlarnaOrderInXt(ArrayObject $kco_order, $xt_order_id, $xt_cart = null)
    {
        global $db;
        $arr = $kco_order->getArrayCopy();
        $json = json_encode($arr, true);
        $db->Execute("UPDATE ".TABLE_ORDERS." SET kco_order_id=?, kco_order_data=?, kco_order_ref=?, kco_order_status=?, kco_order_fraud_status=? WHERE orders_id=?",
            array($kco_order['order_id'], $json, $kco_order['klarna_reference'], $kco_order['status'], $kco_order['fraud_status'], $xt_order_id));
        /*
        if(!empty($xt_cart))
        {
            $db->Execute("UPDATE ".TABLE_ORDERS." SET cart=?",
                array(json_encode($xt_cart));
        }
        */
    }

    public static function getKlarnaOrderFromXt($xt_order_id, $kco_order_id = false)
    {
        global $db;
        $where_col = 'orders_id';
        $params = array($xt_order_id);
        if($kco_order_id != false) {
            $where_col = 'kco_order_id';
            $params = array($kco_order_id);
        }
        $json = $db->GetOne("SELECT kco_order_data FROM ".TABLE_ORDERS." WHERE $where_col=?", $params);
        $ar = json_decode($json,true);
        //$ao = new ArrayObject();
        //$ao->exchangeArray(json_decode($json,true));
        return $ar;
    }

    public static function getKlarnaOrderId($xt_order_id)
    {
        global $db;
        $where_col = 'orders_id';
        $params = array($xt_order_id);

        $klarna_order_id = $db->GetOne("SELECT kco_order_id FROM ".TABLE_ORDERS." WHERE $where_col=?", $params);
        return $klarna_order_id;
    }

    public static function getMappedXtOrderStatus($kco_order_status, $xt_order_store_id)
    {
        global $db;
        $where_col = 'config_key';
        $params = array(strtoupper('_KLARNA_CONFIG_STATUS_'.$kco_order_status));

        $xt_order_status = $db->GetOne("SELECT config_value FROM ".TABLE_CONFIGURATION_MULTI.$xt_order_store_id." WHERE $where_col=?", $params);
        return $xt_order_status;
    }

    public static function setKlarnaOrderIdInXt($kco_order_id, $xt_order_id, $sync = false)
    {
        global $db;
        $db->Execute("UPDATE ".TABLE_ORDERS." SET kco_order_id=? WHERE orders_id=?", array($kco_order_id, $xt_order_id));
        if($sync) self::setXtOrderIdInKlarna($kco_order_id, $xt_order_id);
    }

    public static function setXtOrderIdInKlarna($kco_order_id, $xt_order_id, $sync = false)
    {
        global $shop;
        $domain = !empty($shop->domain) ? $shop->domain : false;

        self::setMerchantReference($kco_order_id, $xt_order_id, $domain);
        if($sync) self::setKlarnaOrderIdInXt($kco_order_id, $xt_order_id);
    }


    public static function createCapture($kco_order_id, $amount, $description = '', array $shipping_info = array(), array $order_lines = array())
    {
        try
        {
            $amount = round($amount, 0);
            $connector = self::getInstance()->getConnector();

            $orderPath = \Klarna\Rest\OrderManagement\Order::$path;
            $capture = new \Klarna\Rest\OrderManagement\Capture($connector, $orderPath.'/'.$kco_order_id);
            $data = array('captured_amount' => $amount, 'description' => $description);
            if(count($shipping_info))
            {
                $data['shipping_info'] = $shipping_info;
            }
            if(count($order_lines))
            {
                $data['order_lines'] = $order_lines;
            }


            $capture->create($data);
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $data, $ex);
            throw $ex;
        }
    }

    public static function addShippingInfo($kco_order_id, $capture_id, array $shipping_infos)
    {
        try
        {
            if (count($shipping_infos))
            {
                $connector = self::getInstance()->getConnector();

                $orderPath = \Klarna\Rest\OrderManagement\Order::$path;
                $capture = new \Klarna\Rest\OrderManagement\Capture($connector, $orderPath . '/' . $kco_order_id, $capture_id );
                $capture->addShippingInfo(array('shipping_info' => $shipping_infos));
            }
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(),
                array('kco_oder_id' => $kco_order_id, 'capture_id' => $capture_id, 'shipping_info' => $shipping_infos),
            $ex);
            throw $ex;
        }
    }

    public static function doRefund($kco_order_id, $amount, $description = '', $order_lines = array())
    {
        try
        {
            $amount = round($amount, 0);
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);
            $data = array('refunded_amount' => $amount, 'description' => $description);
            if(count($order_lines))
            {
                $data['order_lines'] = $order_lines;
            }
            $order->refund($data);
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $data, $ex);
            throw $ex;
        }
    }

    public static function doRelease($kco_order_id)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);

            $order->releaseRemainingAuthorization();
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), array(), $ex);
            throw $ex;
        }
    }

    public static function cancelOrder($kco_order_id)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);

            $order->cancel();
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), array(), $ex);
            throw $ex;
        }
    }

    public static function updateCustomerDetails($kco_order_id, $shipping_address = array(), $billing_address = array())
    {
        try
        {
            $data = array();
            if(is_array($shipping_address) && count($shipping_address))
            {
                $data['shipping_address'] = $shipping_address;
            }
            if(is_array($billing_address) && count($billing_address))
            {
                $data['billing_address'] = $billing_address;
            }
            if(count($data))

            {
                $connector = self::getInstance()->getConnector();

                $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);

                $order->updateCustomerDetails($data);
            }
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), array(), $ex);
            throw $ex;
        }
    }


    public static function acknowledgeOrder($kco_order_id)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);
            $order->acknowledge();
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), array(), $ex);
            throw $ex;
        }
    }


    public static function setMerchantReference($kco_order_id, $ref1, $ref2 = '')
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);
            $data = array(
                'merchant_reference1' => $ref1,
                'merchant_reference2' => $ref2,
            );
            $order->updateMerchantReferences($data);
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $data, $ex);
            throw $ex;
        }
    }


    public static function triggerResendCustomerCommunication($kco_order_id, $capture_id)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);
            $capture = new \Klarna\Rest\OrderManagement\Capture($connector, $order->getLocation(), $capture_id);
            $capture->triggerSendout();
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), array(), $ex);
            throw $ex;
        }
    }

    public static function extendAuthorizationTime($kco_order_id)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);
            $order->extendAuthorizationTime();
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), array(), $ex);
            throw $ex;
        }
    }

    public static function createCheckoutOrder(cart $cart, customer $customer, $customers_status_show_price_tax, array $urls, array &$errors)
    {
        global $tax, $language, $currency, $xtLink, $xtPlugin;

        $order_data = array();
        $order_amount = 0;
        $order_tax_amount = 0;

        $errors = array();

        /**
         * set cart content / line items
         */
        self::setCartContent($cart, $customers_status_show_price_tax,$order_data,  $order_amount, $order_tax_amount);

        /**
         * set cart sub content / fees / discounts ...
         */
        self::setCartSubContent($cart,$order_data,  $order_amount, $order_tax_amount);

        /**
         * set addresses
         *
         */
        self::setAddresses($customer, $order_data);

        /**
         * set customer
         *
         */
        self::setCustomer($customer, $order_data);

        /**
         * set shipping data
         *
         */
        self::setShippingData($customer,$customers_status_show_price_tax,$order_data, $errors);

        $order_data['purchase_currency'] = strtolower($currency->code);

        $order_data['order_amount'] = $order_amount;
        $order_data['order_tax_amount'] = $order_tax_amount;



        /**
         * set language
         */
        self::setLanguageData($language->code, $order_data);

        /**
         * set gui settings
         */
        self::setGuiData($customer, $order_data);

        /**
         * set merchant urls / terms/callback/etc
         */
        self::setMerchantUrls($urls, $order_data);


        self::setPaymentData($customer, $urls, $order_data);

        try
        {
            $connector = self::getInstance()->getConnector();

            $checkout = new \Klarna\Rest\Checkout\Order($connector);
            $checkout->create($order_data);

            $checkout->fetch();
            return $checkout;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $order_data, $ex);
            throw $ex;
        }
    }


    public static function updateOrderAmounts($order, cart $cart, customer $customer, $customers_status_show_price_tax, $urls, $shipping_country, &$errors)
    {
        global $currency;

        try
        {
            $connector = self::getInstance()->getConnector();

            if(is_object($order))
            {}
            else {
                $order = new \Klarna\Rest\Checkout\Order($connector, $order['order_id']);
            }

            $order_data = array();
            $order_amount = 0;
            $order_tax_amount = 0;

            /**
             * set cart content / line items
             */
            self::setCartContent($cart, $customers_status_show_price_tax, $order_data,  $order_amount, $order_tax_amount);

            /**
             * set cart sub content / fees / discounts ...
             */
            self::setCartSubContent($cart,$order_data,  $order_amount, $order_tax_amount);


            /**
             * set gui settings
             */
            self::setGuiData($customer, $order_data);


            /**
             * set addresses
             *
             */
            self::setAddresses($customer, $order_data);

            /**
             * set customer
             *
             */
            self::setCustomer($customer, $order_data);


            /**
             * set shipping data
             *
             */
            self::setShippingData($customer, $customers_status_show_price_tax,$order_data, $errors, $shipping_country);

            self::setPaymentData($customer, $urls, $order_data);

            /**
             * set merchant urls / terms/callback/etc
             */
            self::setMerchantUrls($urls, $order_data);

            $order_data['purchase_currency'] = strtolower($currency->code);

            $order_data['order_amount'] = $order_amount;
            $order_data['order_tax_amount'] = $order_tax_amount;

            $order->update($order_data);

            return $order;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function setLanguage($language_code, $order, $id = false)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            if(is_object($order))
            {}
            else {
                $order = new \Klarna\Rest\Checkout\Order($connector, $id);
            }

            $update_data = array();
            self::setLanguageData($language_code, $update_data);

            $order->update($update_data);
            return $order;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function getOrder($id)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $id);

            $order->fetch();
            return $order;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function getCheckoutOrder($id)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $checkout = new \Klarna\Rest\Checkout\Order($connector, $id);

            $checkout->fetch();
            return $checkout;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function setNewOrderAmount($kco_order_id, $new_total_order_amount, $order_lines)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $order = new \Klarna\Rest\OrderManagement\Order($connector, $kco_order_id);

            $r = $order->updateAuthorization(array('order_amount' => $new_total_order_amount, 'order_lines' => $order_lines));

        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }




    private static function setCartContent($cart, $customers_status_show_price_tax, array &$order_data, &$order_amount, &$order_tax_amount)
    {
        foreach ($cart->show_content as $sc)
        {
            $total_amount       = round($sc['products_final_price']['plain'] * 100,0);
            $total_tax_amount   = round($sc['products_final_tax']['plain'] * 100,0);

            $unit_price = round($sc['products_price']['plain'] * 100,0);

            if($customers_status_show_price_tax == 0)
            {
                $total_amount += $total_tax_amount;

                $unit_price = round($sc['products_price']['plain'] * (1 + $sc['products_tax_info']['tax']/100) * 100,0);
            }



            $total_discount_amount = 0;
            if($sc['_cart_discount'] > 0)
            {
                $unit_price = round ($sc['_original_products_price']['plain'] * 100, 0);
                $total_discount_amount  = $unit_price * $sc['products_quantity'] - $total_amount;
            }

            $product_identifiers = null;
            $identifiers = array();
            if(!empty($sc['products_ean']))
            {
                $identifiers["global_trade_item_number"] = preg_replace('/[^a-zA-Z0-9]/', '' , $sc['products_ean']);
            }
            if(!empty($sc['products_mpn']))
            {
                $identifiers["manufacturer_part_number"] = preg_replace('/[^a-zA-Z0-9]/', '' , $sc['products_mpn']);
            }
            if(count($identifiers))
            {
                $product_identifiers = $identifiers;
            }

            $order_data['order_lines'][] = array(
                "type"          => $sc['products_digital'] == 0 ? "physical" : "digital",
                "reference"     =>  substr($sc['products_model'], 0 , 64),
                "name"          => $sc['products_name'],
                "quantity"      => $sc['products_quantity'],
                "quantity_unit" => $sc['products_unit_name'],
                "unit_price"    => $unit_price,
                "tax_rate"      => round($sc['products_tax_info']['tax'] * 100,0),
                "total_amount"  => $total_amount,
                "total_tax_amount" => $total_tax_amount,
                "total_discount_amount" => $total_discount_amount ,
                "product_identifiers" => $product_identifiers
            );

            if (_KLARNA_CONFIG_SEND_EXTENDED_PRODUCT_INFO == 1)
            {
                $order_data['order_lines'][count($order_data['order_lines'])-1]['product_url'] = $sc['products_link'];

                if(!empty($sc['products_image']))
                {
                    $params = array('img' => $sc['products_image'], 'type' => 'm_info', 'path_only' => true, 'return' => true);
                $img_url = image::getImgUrlOrTag($params, new stdClass());
                $order_data['order_lines'][count($order_data['order_lines'])-1]['image_url'] = $img_url;
            }
            }

            if($sc['manufacturers_id'])
            {
                global $manufacturer;
                $man_data = $manufacturer->getManufacturerData($sc['manufacturers_id']);
                if(!empty($man_data['manufacturers_name']))
                {
                    $order_data['order_lines'][count($order_data['order_lines'])-1]['product_identifiers']['brand'] = $man_data['manufacturers_name'];
                }
            }
            $order_amount       += $total_amount;
            $order_tax_amount   += $total_tax_amount;
        }
    }

    private static function setCartSubContent($cart, array &$order_data, &$order_amount, &$order_tax_amount)
    {
        foreach ($cart->show_sub_content as $k => $sc)
        {
            if(in_array($k, array('shipping', 'payment'))) continue;

            $total_amount       = round($sc['products_final_price']['plain'] * 100,0);
            $total_tax_amount   = round($sc['products_final_tax']['plain'] * 100,0);

            $type = null;
            switch($k)
            {
                case 'xt_coupon':
                    $type = 'discount';
                    break;
                default:
                    // TODO insert hookpoint
            }

            $order_data['order_lines'][] = array(
                "type"          => $type,
                "reference"     =>  substr($sc['products_model'], 0 , 64),
                "name"          => $sc['products_name'],
                "quantity"      => $sc['products_quantity'],
                "quantity_unit" => $sc['products_unit_name'],
                "unit_price"    => round($sc['products_price']['plain'] * 100,0),
                "tax_rate"      => round($sc['products_tax_info']['tax'] * 100,0),
                "total_amount"  => $total_amount,
                "total_tax_amount" => $total_tax_amount
            );
            $order_amount       += $total_amount;
            $order_tax_amount   += $total_tax_amount;
        }
    }

    private static function setCustomer(customer $customer, array &$order_data)
    {
        if ($customer->customers_id && $customer->customer_payment_address['customers_dob'] != false)
        {
            try {
                $format = preg_replace("/([a-zA-z])\\1+/", "$1", _STORE_ACCOUNT_DOB_FORMAT);
                $format = str_replace('y', 'Y', $format);
                $date = DateTime::createFromFormat($format, $customer->customer_payment_address['customers_dob']);
                if(is_object($date))
                {
                    $date_s = $date->format('Y-m-d');
                    $order_data['customer']['date_of_birth'] = $date_s;
                    $order_data['customer']['type'] = 'person';
                }
            }
            catch(Exception $e){}
        }
        else
        {

        }
    }

    private static function setAddresses(customer $customer, array &$order_data, $billing_country_code = false)
    {
        if ($customer->customers_id)
        {
            /**
             * set payment address
             */
            if(isset($_SESSION['kco_login_billing_address']))
            {
                $order_data['billing_address'] = $_SESSION['kco_login_billing_address'];
                $order_data['purchase_country'] = $_SESSION['kco_login_billing_address']['country'];
                unset($_SESSION['kco_login_billing_address']);
            }
            else
            {
            $addr = $customer->customer_payment_address;
            if (is_array($addr) && count($addr))
            {
                    $country_code = strtolower($addr['customers_country_code']);
                    $salut = self::getSalutation($country_code, $addr['customers_gender']);

                $order_data['billing_address'] = array(
                    'title' => $salut,
                    'given_name' => $addr['customers_firstname'],
                    'family_name' => $addr['customers_lastname'],
                    'email' => $_SESSION['customer']->customer_info['customers_email_address'],
                    'street_address' => $addr['customers_street_address'],
                    'postal_code' => $addr['customers_postcode'],
                    'city' => $addr['customers_city'],
                    'region' => $addr['customers_federal_state_code'],
                    'phone' => !empty($addr['customers_mobile_phone']) ? $addr['customers_mobile_phone'] : $addr['customers_phone'],
                        'country' => $country_code
                );
                    $purchase_country = !empty($billing_country_code) ? $billing_country_code : strtolower($addr['customers_country_code']);
                    $order_data['purchase_country'] = $purchase_country;
            }
            }

            if(isset($_SESSION['kco_login_shipping_address']))
            {
                $order_data['shipping_address'] = $_SESSION['kco_login_shipping_address'];
                unset($_SESSION['kco_login_shipping_address']);
            }
            else
            {
            $addr = $customer->customer_shipping_address;
            if (is_array($addr) && count($addr))
            {
                    $country_code = strtolower($addr['customers_country_code']);
                    $salut = self::getSalutation($country_code, $addr['customers_gender']);

                    $order_data['shipping_address'] = array(
                        'title' => $salut,
                        'given_name' => $addr['customers_firstname'],
                        'family_name' => $addr['customers_lastname'],
                        'email' => $_SESSION['customer']->customer_info['customers_email_address'],
                        'street_address' => $addr['customers_street_address'],
                        'postal_code' => $addr['customers_postcode'],
                        'city' => $addr['customers_city'],
                        'region' => $addr['customers_federal_state_code'],
                        'phone' => !empty($addr['customers_mobile_phone']) ? $addr['customers_mobile_phone'] : $addr['customers_phone'],
                        'country' => strtolower($addr['customers_country_code'])
                    );
                }
            }
        }
        else
        {
            // wenn kein angemeldeter nutzer nehmen wir als grundlage store_country
            $order_data['purchase_country'] = strtolower(_STORE_COUNTRY);
        }
    }

    private static function getSalutation($country_code, $gender)
    {
        switch ($gender)
                {
                    case 'f':
                            switch ($country_code)
                            {
                                case 'de':
                                case 'at':
                                case 'ch':
                                    $salut = 'Frau';
                                    break;
                                case 'uk':
                                    $salut = 'Ms';
                                    break;
                                case 'nl':
                                    $salut = 'Mevr.';
                                    break;
                                default:
                                    $salut = 'Mr';
                            }
                        break;
                    case 'm':
                            switch ($country_code)
                            {
                                case 'de':
                                case 'at':
                                case 'ch':
                                    $salut = 'Herr';
                                    break;
                                case 'uk':
                                    $salut = 'Mr';
                                    break;
                                case 'nl':
                                    $salut = 'Dhr.';
                                    break;
                                default:
                                    $salut = 'Mr';
                            }
                        break;
                    case 'c':
                            switch ($country_code)
                            {
                                case 'de':
                                case 'at':
                                case 'ch':
                                    $salut = 'Frau';
                                    break;
                                case 'uk':
                                    $salut = 'Ms';
                                    break;
                                case 'nl':
                                    $salut = 'Mevr.';
                        break;
                    default:
                                    $salut = 'Mr';
                            }
                            break;
                        default:
                            switch ($country_code)
                            {
                                case 'de':
                                case 'at':
                                case 'ch':
                                    $salut = 'Frau';
                                    break;
                                case 'uk':
                                    $salut = 'Ms';
                                    break;
                                case 'nl':
                                    $salut = 'Mevr.';
                                    break;
                                default:
                                    $salut = 'Mr';
                            }
                            break;
                        continue;
                }

        return $salut;
    }

    private static function setPaymentData(customer $customer, $urls, array &$order_data)
    {
        global $tax, $countries;

        $payment_data_in = array();


        if(!$customer->customers_id)
        {
            $country_data = $countries->_getCountryData(_STORE_COUNTRY);
            $payment_data_in['customer_default_address']['customers_country_code'] = _STORE_COUNTRY;
            $payment_data_in['customer_default_address']['customers_zone'] = $country_data['zone_id'];
        }

        $payment = new payment();
        $payment->_payment($payment_data_in);
        if (isset($payment->payment_data))
        {
            foreach ($payment->payment_data as $k => $v)
            {
                $data = self::getPaymentDataForKlarna($v['payment_code']);
                if(!is_array($data) || empty($data)) continue;


                if ($v['payment_price']['discount']==1)
                {
                    $price =  -($order_data['order_amount']*($v['payment_price']['plain']/100));
                }
                else
                {
                    $price = 100.0 * (float)$v['payment_price']['plain'];
                    $price_otax = 100.0 * (float)$v['payment_price']['plain_otax'];
                }

                $order_data['external_payment_methods'][] = [
                    'name' => $data['name'],
                    'redirect_url' => $urls['confirmation_klarna'].'?event=payment&kco_order_id={checkout.order.id}&selected_payment='.$v['payment_code'].'&XDEBUG_SESSION_START='.$_SESSION['XDEBUG_SESSION_START'],
                    'image_url' => $data['imgUrl'],
                    'fee' => $price,
                    'description' => $v['payment_desc']
                ];
            }
        }
    }

    private static function getPaymentDataForKlarna($xt_payment_code)
    {
        $map = array(
            'xt_cashondelivery' => array('name' => 'Nachnahme'),
            'xt_cashpayment' => array('name' => 'Barzahlen'),
            'xt_paypal' => array('name' => 'PayPal'),
            'xt_prepayment' => array('name' => 'Vorkasse'), // filtered by kco
        );

        if(array_key_exists($xt_payment_code, $map))
        {
            global $language;

            $tpl_path = _SRV_WEBROOT._SRV_WEB_TEMPLATES._STORE_TEMPLATE.'/';
            $tpl_path_system = _SRV_WEBROOT._SRV_WEB_TEMPLATES._SYSTEM_TEMPLATE.'/';

            if(_STORE_IMAGES_PATH_FULL == 'true' || true){ // wir brauchen hier die absolute url
                $path_base_url = _SYSTEM_BASE_URL;
            }else{
                $path_base_url = '';
            }
            $tpl_url_path = $path_base_url._SRV_WEB._SRV_WEB_TEMPLATES._STORE_TEMPLATE.'/';
            $tpl_url_path_system = $path_base_url._SRV_WEB._SRV_WEB_TEMPLATES._SYSTEM_TEMPLATE.'/';

            $pathes = array(
                array('path'=> $tpl_path, 'url' => $tpl_url_path),
                array('path'=> $tpl_path_system, 'url' => $tpl_url_path_system)
            );
            $imageUrl = false;

            $lng = $language->content_language;
            if(!in_array($lng, array('de','en')))
            {
                $lng = 'en';
            }
            foreach($pathes as $imgData)
            {
                $imagePath = $imgData['path']."/img/payments-klarna/{$lng}/{$xt_payment_code}.png";
                if(file_exists($imagePath))
                {
                    $imageUrl = $imgData['url']."/img/payments-klarna/{$lng}/{$xt_payment_code}.png";
                    break;
                }
            }
            if($imageUrl)
            {
                $map[$xt_payment_code]['imgUrl'] = $imageUrl;
            }

            return $map[$xt_payment_code];
        }
        return array();
    }

    private static function setShippingData(customer $customer, $customers_status_show_price_tax, array &$order_data, array &$errors, $shipping_country = false)
    {
        global $tax, $countries;

        $shipping_data_in = array();


        if(!$customer->customers_id && !$shipping_country)
        {
            $country_data = $countries->_getCountryData(_STORE_COUNTRY);
            $shipping_data_in['customer_shipping_address']['customers_country_code'] = _STORE_COUNTRY;
            $shipping_data_in['customer_shipping_address']['customers_zone'] = $country_data['zone_id'];
        }
        else if (!empty($shipping_country))
        {
            $country_data = $countries->_getCountryData($shipping_country);
            $shipping_data_in['customer_shipping_address']['customers_country_code'] = $shipping_country;
            $shipping_data_in['customer_shipping_address']['customers_zone'] = $country_data['zone_id'];
        }
        else if (!empty($customer->customers_id))
        {
            $country_data = $countries->_getCountryData($customer->customer_shipping_address['customers_country_code']);
            $shipping_data_in['customer_shipping_address']['customers_country_code'] = $customer->customer_shipping_address['customers_country_code'];
            $shipping_data_in['customer_shipping_address']['customers_zone'] = $country_data['zone_id'];
        }

        $shop_countries = new countries();
        $shop_countries_keys = $shop_countries->_buildCountryList(true,'store', false);
        $shop_countries_keys = array_keys($shop_countries_keys);
        array_walk( $shop_countries_keys, function(&$val){
            $val = strtolower($val);
        } );



        $shipping = new shipping();
        $shipping->_shipping($shipping_data_in);
        if (isset($shipping->shipping_data) && count($shipping->shipping_data))
        {
            foreach ($shipping->shipping_data as $k => $v)
            {
                $total = 100.0*(float)$v['shipping_price']['plain'];
                $tax_amount = 100.0*(float)$v['shipping_price']['plain'] - 100.0*(float)$v['shipping_price']['plain_otax'];

                $tax_rate = $tax->_getTaxRates($v['shipping_tax_class']);

                if($customers_status_show_price_tax == 0)
                {
                    $total_new = $total * (1 + $tax_rate/100);
                    $tax_amount = $total_new - $total;
                    $total = $total_new;
                }

                $order_data['shipping_options'][] = array(
                    'id' => $v['shipping_code'],
                    'name' => $v['shipping_name'],
                    'description' => html_entity_decode(strip_tags($v['shipping_desc'])),
                    'promo' => '',
                    'price' => round($total,0),
                    'tax_amount' => round($tax_amount, 0),
                    'tax_rate' => $tax_rate * 100.0,
                    'preselected' => $_SESSION['kco_selected_shipping'] == $v['shipping_code'] ? true : false
                );
                if($_SESSION['kco_selected_shipping'] == $v['shipping_code'])
                {
                    $_SESSION['selected_shipping'] = $v['shipping_code'];
                }
            }
            if(count($order_data['shipping_options']) && empty($_SESSION['kco_selected_shipping']))
            {
                $order_data['shipping_options'][0]['preselected'] = true;
                $_SESSION['kco_selected_shipping'] = $_SESSION['selected_shipping'] = $order_data['shipping_options'][0]['id'];
            }
        }
        else {
            $err_country = $shipping_country;
            if(empty($err_country))
            {
                $err_country = $shipping_data_in['customer_shipping_address']['customers_country_code'];
            }
            $order_data['shipping_options'][] = array(
                'id' => 'SHIPPING_NOT_AVAILABLE',
                'name' => WARNING_NO_SHIPPING_FOR_ZONE,
                'description' => '',
                'promo' => '',
                'price' => 0,
                'tax_amount' => 0,
                'tax_rate' => 0,
                'preselected' => false
            );

            $errors[] = array('key' => 'NO_SHIPPING_FOUND', 'shipping_country' => $err_country);
        }

        $order_data['shipping_countries'] = $shop_countries_keys;
    }


    private static function setLanguageData($language_code, &$order_data)
    {
        switch($language_code)
        {
            case 'de':
                $order_data['locale'] = 'de-de';
                break;
            case 'en':
                $order_data['locale'] = 'en-gb';
                break;
            case 'da':
                $order_data['locale'] = 'da-dk';
                break;
            case 'fi':
                $order_data['locale'] = 'fi-fi';
                break;
            case 'nl':
                $order_data['locale'] = 'nl-nl';
                break;
            case 'nb':
                $order_data['locale'] = 'nb-no';
                break;
            case 'sv':
                $order_data['locale'] = 'sv-se';
                break;
            default:
                $order_data['locale'] = 'en-gb';
        }
    }


    private static function setGuiData(customer $customer, array &$order_data)
    {
        $order_data['options']['allow_separate_shipping_address'] = _KLARNA_CONFIG_ALLOW_SEPARATE_SHIPPING_ADDRESS == 1 ? true : false;

        $order_data['options']['radius_border'] = _KLARNA_CONFIG_RADIUS_BORDER;

        $order_data['options']['color_button'] = _KLARNA_CONFIG_COLOR_BUTTON;
        $order_data['options']['color_button_text'] = _KLARNA_CONFIG_COLOR_BUTTON_TEXT;
        $order_data['options']['color_checkbox'] = _KLARNA_CONFIG_COLOR_CHECKBOX;
        $order_data['options']['color_checkbox_checkmark'] = _KLARNA_CONFIG_COLOR_CHECKBOX_CHECKMARK;
        $order_data['options']['color_header'] = _KLARNA_CONFIG_COLOR_HEADER;
        $order_data['options']['color_link'] = _KLARNA_CONFIG_COLOR_LINK;

        $order_data['options']['date_of_birth_mandatory'] = _STORE_ACCOUNT_DOB == 'true' ? true : false;
        $order_data['options']['title_mandatory'] = _STORE_ACCOUNT_GENDER == 'true' ? true : false;

        $order_data['options']['require_validate_callback_success'] = false;

        $order_data['options']['show_subtotal_detail'] = true;
        $order_data['gui']['options'] = array('disable_autofocus', 'minimal_confirmation');

        if(_KLARNA_CONFIG_REGISTER_CHECKBOX == 1 && empty($customer->customers_id))
        {
            $order_data['options']['additional_checkbox'] = array(
                'required' => false,
                'checked' => true,
                'text' => TEXT_I_WANT_TO_REGISTER
            );
        }
    }


    private static function setMerchantUrls($urls, &$order_data)
    {
        $debug = '';
        if(!empty($_SESSION['XDEBUG_SESSION_START']))
        {
            $debug = '&XDEBUG_SESSION_START='.$_SESSION['XDEBUG_SESSION_START'];

        }
        $sid = '&registered_customer='. ( empty($_SESSION['registered_customer']) ? 0 : $_SESSION['registered_customer']);

        $merchantUrls = [
            'terms'                 => $urls['terms'],
            'cancellation_terms'    => $urls['cancellation_terms'],

            'checkout'              =>  $urls['confirmation_shop'].'?event=checkout&kco_order_id={checkout.order.id}'.$debug,
            'confirmation'          =>  $urls['confirmation_klarna'].'?event=confirmation&kco_order_id={checkout.order.id}'.$debug,

            'push'                  =>  $urls['push'].'&event=push&kco_order_id={checkout.order.id}'.$debug,
            'notification'          =>  $urls['push'].'&event=notification&kco_order_id={checkout.order.id}'.$debug,
            'validation'            =>  $urls['push'].'&event=validation&kco_order_id={checkout.order.id}'.$sid.$debug,

            /*
            'address_update'            => $urls['push'].'&kco_order_id={checkout.order.id}&event=address_update&XDEBUG_SESSION_START='.$_SESSION['XDEBUG_SESSION_START'],
            'country_change'            => $urls['push'].'&kco_order_id={checkout.order.id}&event=country_change&XDEBUG_SESSION_START='.$_SESSION['XDEBUG_SESSION_START'],
            'shipping_option_update'    => $urls['push'].'&kco_order_id={checkout.order.id}&event=shipping_option_update&XDEBUG_SESSION_START='.$_SESSION['XDEBUG_SESSION_START'],

            'shipping_address_update'    => $urls['push'].'&kco_order_id={checkout.order.id}&event=shipping_address_update&XDEBUG_SESSION_START='.$_SESSION['XDEBUG_SESSION_START']
                */
        ];
        $order_data['merchant_urls'] = $merchantUrls;
    }

    /**
     *    SETTLEMENTS API
     */

    /**  Payouts */
    public static function getPayoutSummary($start_date, $end_date, $currency_code = null)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $payout = new \Klarna\Rest\Settlements\Payouts($connector);

            $payout->getSummary($start_date , $end_date , $currency_code);
            return $payout;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function getPayout($payment_reference)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $payout = new \Klarna\Rest\Settlements\Payouts($connector);

            $payout->getPayout($payment_reference);
            return $payout;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function getPayouts($offset = 0, $size = 0, $start_date = null, $end_date = null, $currency_code = null)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $payout = new \Klarna\Rest\Settlements\Payouts($connector);

            $payout->page($offset, $size, $start_date, $end_date, $currency_code);
            return $payout;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }



    /**  Transactions */
    public static function getTransactions($payment_reference)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $transaction = new \Klarna\Rest\Settlements\Transactions($connector);

            $transaction->getTransactions($payment_reference);
            return $transaction;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function getAllTransactions($offset = 0, $size = 0, $store_id)
    {
        try
        {
            $connector = self::getInstance()->getConnector($store_id);

            $transaction = new \Klarna\Rest\Settlements\Transactions($connector);

            $transaction->page($offset, $size);
            return $transaction;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }


    /**  Reports */
    public static function getPayoutReport($payment_reference)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $report = new \Klarna\Rest\Settlements\Reports($connector);

            $report->getPayoutWithTransactions($payment_reference);
            return $report;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    public static function getPayoutsSummaryReport($start_date, $end_date)
    {
        try
        {
            $connector = self::getInstance()->getConnector();

            $report = new \Klarna\Rest\Settlements\Reports($connector);

            $report->getPayoutsSummaryWithTransactions($start_date , $end_date);
            return $report;
        }
        catch(Exception $ex)
        {
            self::log(__FUNCTION__, func_get_args(), $ex);
            throw $ex;
        }
    }

    /**
     *    HELPER FUNCTIONS
     */


    public static function compareAddresses(array $a1, array $a2, $ignore = array())
    {

        foreach($a1 as $k => $v)
        {
            if (in_array($k, $ignore)) continue;
            if ($a1[$k] != $a2[$k]) return false;
        }
        return true;
    }


    public static function log($fnc, $xt_data = '', $kco_data = array(), Exception $ex = null)
    {
        if(!empty($ex) || (defined('_KCO_LOG') && _KCO_LOG === true))
        {
            $logFile = _SRV_WEBROOT . 'xtLogs/klarna_kco.log';
            if (file_exists($logFile) && filesize($logFile) > 10485760)
            {
                rename($logFile, _SRV_WEBROOT . 'xtLogs/klarna_kco.' . date('Y-m-d_H-i-s') . '.log');
            }
            $f = fopen(_SRV_WEBROOT . 'xtLogs/klarna_kco.log', 'a+');
            if ($f)
            {
                $s = date("[Y-m-d H:i:s]") . '  ' . $fnc . PHP_EOL;
                if (is_array($xt_data))
                {
                    $xt_data = print_r($xt_data, true);
                    $s .= 'xt data => ' . $xt_data;
                }
                else
                {
                    $s .= 'xt data => ' . $xt_data . PHP_EOL;
                }
                if(!empty($kco_data))
                {
                    $kco_data = print_r($kco_data, true);
                    $s .= 'kco data => ' . $kco_data;
                }

                if (!empty($ex))
                {
                    $s .= 'Exception => ' . $ex->getMessage() . PHP_EOL;
                    $s .= $ex->getTraceAsString() . PHP_EOL;
                }
                fwrite($f, $s);
                fclose($f);
            }
        }
    }

    public static function convert_country_code( $country, $iso3toIso2=false)
    {
        $countries = array(
            'AF' => 'AFG', //Afghanistan
            'AX' => 'ALA', //&#197;land Islands
            'AL' => 'ALB', //Albania
            'DZ' => 'DZA', //Algeria
            'AS' => 'ASM', //American Samoa
            'AD' => 'AND', //Andorra
            'AO' => 'AGO', //Angola
            'AI' => 'AIA', //Anguilla
            'AQ' => 'ATA', //Antarctica
            'AG' => 'ATG', //Antigua and Barbuda
            'AR' => 'ARG', //Argentina
            'AM' => 'ARM', //Armenia
            'AW' => 'ABW', //Aruba
            'AU' => 'AUS', //Australia
            'AT' => 'AUT', //Austria
            'AZ' => 'AZE', //Azerbaijan
            'BS' => 'BHS', //Bahamas
            'BH' => 'BHR', //Bahrain
            'BD' => 'BGD', //Bangladesh
            'BB' => 'BRB', //Barbados
            'BY' => 'BLR', //Belarus
            'BE' => 'BEL', //Belgium
            'BZ' => 'BLZ', //Belize
            'BJ' => 'BEN', //Benin
            'BM' => 'BMU', //Bermuda
            'BT' => 'BTN', //Bhutan
            'BO' => 'BOL', //Bolivia
            'BQ' => 'BES', //Bonaire, Saint Estatius and Saba
            'BA' => 'BIH', //Bosnia and Herzegovina
            'BW' => 'BWA', //Botswana
            'BV' => 'BVT', //Bouvet Islands
            'BR' => 'BRA', //Brazil
            'IO' => 'IOT', //British Indian Ocean Territory
            'BN' => 'BRN', //Brunei
            'BG' => 'BGR', //Bulgaria
            'BF' => 'BFA', //Burkina Faso
            'BI' => 'BDI', //Burundi
            'KH' => 'KHM', //Cambodia
            'CM' => 'CMR', //Cameroon
            'CA' => 'CAN', //Canada
            'CV' => 'CPV', //Cape Verde
            'KY' => 'CYM', //Cayman Islands
            'CF' => 'CAF', //Central African Republic
            'TD' => 'TCD', //Chad
            'CL' => 'CHL', //Chile
            'CN' => 'CHN', //China
            'CX' => 'CXR', //Christmas Island
            'CC' => 'CCK', //Cocos (Keeling) Islands
            'CO' => 'COL', //Colombia
            'KM' => 'COM', //Comoros
            'CG' => 'COG', //Congo
            'CD' => 'COD', //Congo, Democratic Republic of the
            'CK' => 'COK', //Cook Islands
            'CR' => 'CRI', //Costa Rica
            'CI' => 'CIV', //Côte d\'Ivoire
            'HR' => 'HRV', //Croatia
            'CU' => 'CUB', //Cuba
            'CW' => 'CUW', //Curaçao
            'CY' => 'CYP', //Cyprus
            'CZ' => 'CZE', //Czech Republic
            'DK' => 'DNK', //Denmark
            'DJ' => 'DJI', //Djibouti
            'DM' => 'DMA', //Dominica
            'DO' => 'DOM', //Dominican Republic
            'EC' => 'ECU', //Ecuador
            'EG' => 'EGY', //Egypt
            'SV' => 'SLV', //El Salvador
            'GQ' => 'GNQ', //Equatorial Guinea
            'ER' => 'ERI', //Eritrea
            'EE' => 'EST', //Estonia
            'ET' => 'ETH', //Ethiopia
            'FK' => 'FLK', //Falkland Islands
            'FO' => 'FRO', //Faroe Islands
            'FJ' => 'FIJ', //Fiji
            'FI' => 'FIN', //Finland
            'FR' => 'FRA', //France
            'GF' => 'GUF', //French Guiana
            'PF' => 'PYF', //French Polynesia
            'TF' => 'ATF', //French Southern Territories
            'GA' => 'GAB', //Gabon
            'GM' => 'GMB', //Gambia
            'GE' => 'GEO', //Georgia
            'DE' => 'DEU', //Germany
            'GH' => 'GHA', //Ghana
            'GI' => 'GIB', //Gibraltar
            'GR' => 'GRC', //Greece
            'GL' => 'GRL', //Greenland
            'GD' => 'GRD', //Grenada
            'GP' => 'GLP', //Guadeloupe
            'GU' => 'GUM', //Guam
            'GT' => 'GTM', //Guatemala
            'GG' => 'GGY', //Guernsey
            'GN' => 'GIN', //Guinea
            'GW' => 'GNB', //Guinea-Bissau
            'GY' => 'GUY', //Guyana
            'HT' => 'HTI', //Haiti
            'HM' => 'HMD', //Heard Island and McDonald Islands
            'VA' => 'VAT', //Holy See (Vatican City State)
            'HN' => 'HND', //Honduras
            'HK' => 'HKG', //Hong Kong
            'HU' => 'HUN', //Hungary
            'IS' => 'ISL', //Iceland
            'IN' => 'IND', //India
            'ID' => 'IDN', //Indonesia
            'IR' => 'IRN', //Iran
            'IQ' => 'IRQ', //Iraq
            'IE' => 'IRL', //Republic of Ireland
            'IM' => 'IMN', //Isle of Man
            'IL' => 'ISR', //Israel
            'IT' => 'ITA', //Italy
            'JM' => 'JAM', //Jamaica
            'JP' => 'JPN', //Japan
            'JE' => 'JEY', //Jersey
            'JO' => 'JOR', //Jordan
            'KZ' => 'KAZ', //Kazakhstan
            'KE' => 'KEN', //Kenya
            'KI' => 'KIR', //Kiribati
            'KP' => 'PRK', //Korea, Democratic People\'s Republic of
            'KR' => 'KOR', //Korea, Republic of (South)
            'KW' => 'KWT', //Kuwait
            'KG' => 'KGZ', //Kyrgyzstan
            'LA' => 'LAO', //Laos
            'LV' => 'LVA', //Latvia
            'LB' => 'LBN', //Lebanon
            'LS' => 'LSO', //Lesotho
            'LR' => 'LBR', //Liberia
            'LY' => 'LBY', //Libya
            'LI' => 'LIE', //Liechtenstein
            'LT' => 'LTU', //Lithuania
            'LU' => 'LUX', //Luxembourg
            'MO' => 'MAC', //Macao S.A.R., China
            'MK' => 'MKD', //Macedonia
            'MG' => 'MDG', //Madagascar
            'MW' => 'MWI', //Malawi
            'MY' => 'MYS', //Malaysia
            'MV' => 'MDV', //Maldives
            'ML' => 'MLI', //Mali
            'MT' => 'MLT', //Malta
            'MH' => 'MHL', //Marshall Islands
            'MQ' => 'MTQ', //Martinique
            'MR' => 'MRT', //Mauritania
            'MU' => 'MUS', //Mauritius
            'YT' => 'MYT', //Mayotte
            'MX' => 'MEX', //Mexico
            'FM' => 'FSM', //Micronesia
            'MD' => 'MDA', //Moldova
            'MC' => 'MCO', //Monaco
            'MN' => 'MNG', //Mongolia
            'ME' => 'MNE', //Montenegro
            'MS' => 'MSR', //Montserrat
            'MA' => 'MAR', //Morocco
            'MZ' => 'MOZ', //Mozambique
            'MM' => 'MMR', //Myanmar
            'NA' => 'NAM', //Namibia
            'NR' => 'NRU', //Nauru
            'NP' => 'NPL', //Nepal
            'NL' => 'NLD', //Netherlands
            'AN' => 'ANT', //Netherlands Antilles
            'NC' => 'NCL', //New Caledonia
            'NZ' => 'NZL', //New Zealand
            'NI' => 'NIC', //Nicaragua
            'NE' => 'NER', //Niger
            'NG' => 'NGA', //Nigeria
            'NU' => 'NIU', //Niue
            'NF' => 'NFK', //Norfolk Island
            'MP' => 'MNP', //Northern Mariana Islands
            'NO' => 'NOR', //Norway
            'OM' => 'OMN', //Oman
            'PK' => 'PAK', //Pakistan
            'PW' => 'PLW', //Palau
            'PS' => 'PSE', //Palestinian Territory
            'PA' => 'PAN', //Panama
            'PG' => 'PNG', //Papua New Guinea
            'PY' => 'PRY', //Paraguay
            'PE' => 'PER', //Peru
            'PH' => 'PHL', //Philippines
            'PN' => 'PCN', //Pitcairn
            'PL' => 'POL', //Poland
            'PT' => 'PRT', //Portugal
            'PR' => 'PRI', //Puerto Rico
            'QA' => 'QAT', //Qatar
            'RE' => 'REU', //Reunion
            'RO' => 'ROU', //Romania
            'RU' => 'RUS', //Russia
            'RW' => 'RWA', //Rwanda
            'BL' => 'BLM', //Saint Barth&eacute;lemy
            'SH' => 'SHN', //Saint Helena
            'KN' => 'KNA', //Saint Kitts and Nevis
            'LC' => 'LCA', //Saint Lucia
            'MF' => 'MAF', //Saint Martin (French part)
            'SX' => 'SXM', //Sint Maarten / Saint Matin (Dutch part)
            'PM' => 'SPM', //Saint Pierre and Miquelon
            'VC' => 'VCT', //Saint Vincent and the Grenadines
            'WS' => 'WSM', //Samoa
            'SM' => 'SMR', //San Marino
            'ST' => 'STP', //S&atilde;o Tom&eacute; and Pr&iacute;ncipe
            'SA' => 'SAU', //Saudi Arabia
            'SN' => 'SEN', //Senegal
            'RS' => 'SRB', //Serbia
            'SC' => 'SYC', //Seychelles
            'SL' => 'SLE', //Sierra Leone
            'SG' => 'SGP', //Singapore
            'SK' => 'SVK', //Slovakia
            'SI' => 'SVN', //Slovenia
            'SB' => 'SLB', //Solomon Islands
            'SO' => 'SOM', //Somalia
            'ZA' => 'ZAF', //South Africa
            'GS' => 'SGS', //South Georgia/Sandwich Islands
            'SS' => 'SSD', //South Sudan
            'ES' => 'ESP', //Spain
            'LK' => 'LKA', //Sri Lanka
            'SD' => 'SDN', //Sudan
            'SR' => 'SUR', //Suriname
            'SJ' => 'SJM', //Svalbard and Jan Mayen
            'SZ' => 'SWZ', //Swaziland
            'SE' => 'SWE', //Sweden
            'CH' => 'CHE', //Switzerland
            'SY' => 'SYR', //Syria
            'TW' => 'TWN', //Taiwan
            'TJ' => 'TJK', //Tajikistan
            'TZ' => 'TZA', //Tanzania
            'TH' => 'THA', //Thailand
            'TL' => 'TLS', //Timor-Leste
            'TG' => 'TGO', //Togo
            'TK' => 'TKL', //Tokelau
            'TO' => 'TON', //Tonga
            'TT' => 'TTO', //Trinidad and Tobago
            'TN' => 'TUN', //Tunisia
            'TR' => 'TUR', //Turkey
            'TM' => 'TKM', //Turkmenistan
            'TC' => 'TCA', //Turks and Caicos Islands
            'TV' => 'TUV', //Tuvalu
            'UG' => 'UGA', //Uganda
            'UA' => 'UKR', //Ukraine
            'AE' => 'ARE', //United Arab Emirates
            'GB' => 'GBR', //United Kingdom
            'US' => 'USA', //United States
            'UM' => 'UMI', //United States Minor Outlying Islands
            'UY' => 'URY', //Uruguay
            'UZ' => 'UZB', //Uzbekistan
            'VU' => 'VUT', //Vanuatu
            'VE' => 'VEN', //Venezuela
            'VN' => 'VNM', //Vietnam
            'VG' => 'VGB', //Virgin Islands, British
            'VI' => 'VIR', //Virgin Island, U.S.
            'WF' => 'WLF', //Wallis and Futuna
            'EH' => 'ESH', //Western Sahara
            'YE' => 'YEM', //Yemen
            'ZM' => 'ZMB', //Zambia
            'ZW' => 'ZWE', //Zimbabwe
        );

        if($iso3toIso2)
        {
            $countries = array_flip($countries);
        }
        $iso_code = isset( $countries[$country] ) ? $countries[$country] : $country;
        return $iso_code;
    }

}

class NoShippingFoundException extends Exception{

}


